@@ -1,18 +1,18 @@
 opm_files <- function(what = c("scripts", "testdata", "auxiliary", "demo",
-    "examples", "doc", "css", "omnilog", "single", "multiple")) {
-  switch(match.arg(what),
+    "doc", "css", "sql", "omnilog", "single", "multiple", "growth")) {
+  switch(what <- match.arg(what),
     css = grep("\\.css$", pkg_files(opm_string(), "auxiliary"),
       TRUE, TRUE, TRUE),
-    examples = {
-      warning("'examples' is deprecated, use 'demo'")
-      pkg_files(opm_string(), "demo")
-    },
+    growth = grep("\\.asc(\\.[^.]+)?$",
+      pkg_files(opm_string(), "testdata"), TRUE, TRUE, TRUE),
     multiple = grep("Multiple\\.csv(\\.[^.]+)?$",
       pkg_files(opm_string(), "testdata"), TRUE, TRUE, TRUE),
     omnilog = grep("Example(_Old_Style)?_\\d+\\.csv(\\.[^.]+)?$",
       pkg_files(opm_string(), "testdata"), TRUE, TRUE, TRUE),
     single = grep("Multiple\\.csv(\\.[^.]+)?$", pkg_files(opm_string(),
       "testdata"), TRUE, TRUE, TRUE, FALSE, FALSE, TRUE),
+    sql = grep("\\.sql$", pkg_files(opm_string(), "auxiliary"),
+      TRUE, TRUE, TRUE),
     pkg_files(opm_string(), what)
   )
 }
@@ -66,6 +66,89 @@ select_colors <- function(
   )
 }
 
+custom_plate_is <- function(x) grepl("^Custom:", x, TRUE, TRUE)
+
+custom_plate_proper <- function(x) substring(x, 8L, nchar(x))
+
+custom_plate_prepend <- function(x) sprintf("CUSTOM:%s", x)
+
+custom_plate_prepend_full <- function(x) sprintf("CUSTOM_FULL_NAME:%s", x)
+
+custom_plate_normalize_proper <- function(x) {
+  x <- sub("\\W+$", "", sub("^\\W+", "", x, FALSE, TRUE), FALSE, TRUE)
+  toupper(gsub("\\W+", "-", x, FALSE, TRUE))
+}
+
+custom_plate_normalize <- function(x) {
+  custom_plate_prepend(custom_plate_normalize_proper(custom_plate_proper(x)))
+}
+
+custom_plate_normalize_all <- function(x) {
+  x <- ifelse(custom_plate_is(x), custom_plate_proper(x), x)
+  custom_plate_prepend(custom_plate_normalize_proper(x))
+}
+
+custom_plate_exists <- function(x) {
+  exists(x, MEMOIZED)
+}
+
+custom_plate_get <- function(x) {
+  get(x, MEMOIZED)
+}
+
+custom_plate_assert <- function(x, coords) {
+  if (custom_plate_exists(x)) {
+    if (any(bad <- !coords %in% names(custom_plate_get(x))))
+      stop("well coordinate missing from plate type '", x, "': ",
+        coords[bad][1L])
+  } else
+    stop("unknown user-defined plate type: ", x)
+  TRUE
+}
+
+custom_plate_set <- function(x, value) {
+  if (exists(x, MEMOIZED))
+    warning("overwriting well map for plate type ", x)
+  MEMOIZED[[x]] <- value
+  value
+}
+
+custom_plate_set_full <- function(x, value) {
+  key <- custom_plate_prepend_full(custom_plate_proper(x))
+  names(value) <- NULL
+  if (exists(key, MEMOIZED) && !identical(value, get(key, MEMOIZED)))
+    warning("overwriting full name for plate type ", x)
+  MEMOIZED[[key]] <- value
+  value
+}
+
+normalize_predefined_plate <- function(object, subtype = FALSE) {
+  normalize_pm <- function(x, subtype) {
+    x <- sub("^PMM", "PM-M", x, FALSE, TRUE)
+    x <- sub("^PM-MTOX", "PM-M TOX", x, FALSE, TRUE)
+    x <- sub("([A-Z]+)$", if (subtype)
+      "-\\1"
+    else
+      "", x, FALSE, TRUE)
+    sub("([^\\d])(\\d)([^\\d]|$)", "\\10\\2\\3", x, FALSE, TRUE)
+  }
+  normalize_sf <- function(x, subtype) {
+    x <- if (subtype)
+      sub("-$", "", sub(SP_PATTERN, "\\1-\\2", x, FALSE, TRUE), FALSE, TRUE)
+    else
+      sub(SP_PATTERN, "\\1", x, FALSE, TRUE)
+    x <- sub("^(G|SF)([NP])", "SF-\\2", x, FALSE, TRUE)
+    sub("^GENIII", "Gen III", x, FALSE, TRUE)
+  }
+  result <- toupper(gsub("\\W", "", object, FALSE, TRUE))
+  pm <- grepl("^PM(M(TOX)?)?\\d+[A-Z]*$", result, FALSE, TRUE)
+  result[pm] <- normalize_pm(result[pm], subtype)
+  sf[sf] <- grepl(SP_PATTERN, result[sf <- !pm], FALSE, TRUE)
+  result[sf] <- normalize_sf(result[sf], subtype)
+  result[bad] <- object[bad <- !(pm | sf)]
+  result
+}
+
 setGeneric("plate_type", function(object, ...) standardGeneric("plate_type"))
 
 setMethod("plate_type", OPM, function(object, ..., normalize = FALSE,
@@ -74,35 +157,57 @@ setMethod("plate_type", OPM, function(object, ..., normalize = FALSE,
     normalize = normalize, subtype = subtype)
 }, sealed = SEALED)
 
+setMethod("plate_type", MOPMX, function(object, ..., normalize = FALSE,
+    subtype = FALSE) {
+  vapply(X = object@.Data, FUN = plate_type, FUN.VALUE = "", ...,
+    normalize = normalize, subtype = subtype)
+}, sealed = SEALED)
+
+setMethod("plate_type", "OPM_DB", function(object, ..., normalize = FALSE,
+    subtype = FALSE) {
+  plate_type(object = object@plates[, "plate_type"], ...,
+    normalize = normalize, subtype = subtype)
+}, sealed = SEALED)
+
 setMethod("plate_type", "character", function(object, full = FALSE,
     in.parens = TRUE, max = opm_opt("max.chars"), clean = TRUE,
     brackets = FALSE, word.wise = FALSE, paren.sep = " ", downcase = FALSE,
     normalize = TRUE, subtype = FALSE) {
   do_normalize <- function(object, subtype) {
-    normalize_pm <- function(x, subtype) {
-      x <- sub("^PMM", "PM-M", x, FALSE, TRUE)
-      x <- sub("^PM-MTOX", "PM-M TOX", x, FALSE, TRUE)
-      x <- sub("([A-Z]+)$", if (subtype)
-        "-\\1"
-      else
-        "", x, FALSE, TRUE)
-      sub("([^\\d])(\\d)([^\\d]|$)", "\\10\\2\\3", x, FALSE, TRUE)
-    }
-    normalize_sf <- function(x, subtype) {
-      x <- if (subtype)
-        sub("-$", "", sub(SP_PATTERN, "\\1-\\2", x, FALSE, TRUE), FALSE, TRUE)
-      else
-        sub(SP_PATTERN, "\\1", x, FALSE, TRUE)
-      x <- sub("^(G|SF)([NP])", "SF-\\2", x, FALSE, TRUE)
-      sub("^GENIII", "Gen III", x, FALSE, TRUE)
-    }
-    result <- toupper(gsub("\\W", "", object, FALSE, TRUE))
-    pm <- grepl("^PM(M(TOX)?)?\\d+[A-Z]*$", result, FALSE, TRUE)
-    result[pm] <- normalize_pm(result[pm], subtype)
-    sf[sf] <- grepl(SP_PATTERN, result[sf <- !pm], FALSE, TRUE)
-    result[sf] <- normalize_sf(result[sf], subtype)
-    result[bad] <- object[bad <- !(pm | sf)]
-    result
+    is.custom <- custom_plate_is(object)
+    object[!is.custom] <- normalize_predefined_plate(object[!is.custom],
+      subtype)
+    object[is.custom] <- custom_plate_normalize(object[is.custom])
+    object
+  }
+  orig_and_full <- function(orig, full.name) {
+    if (downcase)
+      full.name <- substrate_info(full.name, "downcase")
+    if (in.parens)
+      add_in_parens(str.1 = orig, str.2 = full.name, max = max,
+        clean = clean, brackets = brackets, word.wise = word.wise,
+        paren.sep = paren.sep)
+    else
+      trim_string(str = full.name, max = max, clean = clean,
+        word.wise = word.wise)
+  }
+  expand_predefined <- function(x) {
+    pos <- match(x, names(PLATE_MAP))
+    ok <- !is.na(pos)
+    for (name in x[!ok])
+      warning("cannot find full name of plate ", name)
+    x[ok] <- orig_and_full(x[ok], PLATE_MAP[pos[ok]])
+    x
+  }
+  expand_custom <- function(x) {
+    if (!length(x))
+      return(x)
+    n <- custom_plate_prepend_full(custom_plate_proper(x))
+    ok <- vapply(n, exists, NA, MEMOIZED)
+    for (name in x[!ok])
+      warning("cannot find full name of plate ", name)
+    x[ok] <- orig_and_full(x[ok], unlist(mget(n[ok], MEMOIZED), FALSE, FALSE))
+    x
   }
   LL(full, downcase, in.parens, normalize, subtype)
   result <- if (normalize)
@@ -111,21 +216,9 @@ setMethod("plate_type", "character", function(object, full = FALSE,
     object
   if (!full)
     return(result)
-  pos <- match(result, names(PLATE_MAP))
-  if (any(bad <- is.na(pos))) {
-    warning("cannot find full name of plate ", result[bad][1L])
-    return(result)
-  }
-  full.name <- PLATE_MAP[pos]
-  if (downcase)
-    full.name <- substrate_info(full.name, "downcase")
-  if (in.parens)
-    result <- add_in_parens(str.1 = result, str.2 = full.name, max = max,
-      clean = clean, brackets = brackets, word.wise = word.wise,
-      paren.sep = paren.sep)
-  else
-    result <- trim_string(str = full.name, max = max, clean = clean,
-      word.wise = word.wise)
+  is.custom <- custom_plate_is(result)
+  result[!is.custom] <- expand_predefined(result[!is.custom])
+  result[is.custom] <- expand_custom(result[is.custom])
   result
 }, sealed = SEALED)
 
@@ -134,14 +227,32 @@ setMethod("plate_type", "factor", function(object, ...) {
 }, sealed = SEALED)
 
 setMethod("plate_type", "missing", function(object, ...) {
-  plate_type(names(PLATE_MAP), ...)
+  x <- ls(MEMOIZED)
+  plate_type(c(names(PLATE_MAP), x[custom_plate_is(x)]), ...)
+}, sealed = SEALED)
+
+setMethod("plate_type", "logical", function(object, ...) {
+  if (is.na(L(object))) {
+    x <- ls(MEMOIZED)
+    x <- c(names(PLATE_MAP), x[custom_plate_is(x)])
+  } else if (object) {
+    x <- ls(MEMOIZED)
+    x <- x[custom_plate_is(x)]
+  } else {
+    x <- names(PLATE_MAP)
+  }
+  plate_type(x, ...)
 }, sealed = SEALED)
 
 setGeneric("gen_iii", function(object, ...) standardGeneric("gen_iii"))
 
 setMethod("gen_iii", OPM, function(object, to = "gen.iii") {
-  to <- match.arg(tolower(to), names(SPECIAL_PLATES))
-  object@csv_data[[CSV_NAMES[["PLATE_TYPE"]]]] <- SPECIAL_PLATES[[to]]
+  if (custom_plate_is(L(to))) {
+    to <- custom_plate_normalize(to)
+    custom_plate_assert(to, colnames(object@measurements)[-1L])
+  } else
+    to <- SPECIAL_PLATES[[match.arg(tolower(to), names(SPECIAL_PLATES))]]
+  object@csv_data[[CSV_NAMES[["PLATE_TYPE"]]]] <- to
   object
 }, sealed = SEALED)
 
@@ -151,10 +262,88 @@ setMethod("gen_iii", OPMS, function(object, ...) {
 }, sealed = SEALED)
 
 setMethod("gen_iii", MOPMX, function(object, ...) {
-  object@.Data <- lapply(X = object@.Data, FUN = gen_iii, ...)
+  object@.Data <- mapply(FUN = gen_iii, object = object@.Data, ...,
+    MoreArgs = NULL, SIMPLIFY = FALSE, USE.NAMES = FALSE)
   object
 }, sealed = SEALED)
 
+setGeneric("register_plate",
+  function(object, ...) standardGeneric("register_plate"))
+
+setMethod("register_plate", "character", function(object, ...) {
+  x <- do.call(c, lapply(object, function(file) tryCatch(yaml.load_file(file),
+    error = function(e) readRDS(file))))
+  x <- mapply(FUN = function(d, n) {
+      names(d) <- rep.int(n, length(d))
+      d
+    }, d = x, n = names(x), SIMPLIFY = FALSE, USE.NAMES = FALSE)
+  register_plate(do.call(c, x), ...)
+}, sealed = SEALED)
+
+setMethod("register_plate", "missing", function(object, ...) {
+  register_plate(list(...))
+}, sealed = SEALED)
+
+setMethod("register_plate", "list", function(object, ...) {
+  valid_names <- function(n) length(n) && !any(is.na(n)) && all(nzchar(n))
+  prepare_names <- function(n) {
+    if (!valid_names(n))
+      stop("all arguments must be validly named")
+    n <- ifelse(custom_plate_is(n), custom_plate_proper(n), n)
+    custom_plate_normalize_proper(n)
+  }
+  convert_rectangular_coords <- function(x) {
+    if (!length(j <- as.integer(colnames(x))))
+      j <- seq_len(ncol(x))
+    if (!length(i <- rownames(x)))
+      i <- rep(LETTERS, length.out = nrow(x))
+    n <- vapply(i, sprintf, character(length(j)), fmt = "%s%02i", j)
+    structure(c(t(x)), names = c(n))
+  }
+  prepare_well_map <- function(x) {
+    if (inherits(x, "well_coords_map")) {
+      case(ncol(x), stop("'well_coords_map' object has zero columns"),
+        x <- x[, 1L], {
+          warning("discarding additional columns in 'well_coords_map' object")
+          x <- x[, 1L]
+        })
+    } else if (is.data.frame(x)) {
+      for (i in which(vapply(x, is.factor, NA)))
+        x[, i] <- as.character(x[, i])
+      x <- convert_rectangular_coords(as.matrix(x))
+    } else if (is.matrix(x)) {
+      x <- convert_rectangular_coords(x)
+    } else {
+      names(x) <- clean_coords(names(x))
+    }
+    storage.mode(x) <- "character"
+    if (dup <- anyDuplicated(names(x)))
+      stop("duplicate well coordinate provided: ", names(x)[dup])
+    x
+  }
+  insert_plate_types <- function(x) {
+    named <- vapply(lapply(x, names), valid_names, NA) |
+      vapply(x, is.data.frame, NA) | vapply(x, is.matrix, NA)
+    if (any(vapply(x, length, 0L) > 1L & !named))
+      stop("element unnamed but not of length 1")
+    x[named] <- lapply(x[named], prepare_well_map)
+    names(x) <- ifelse(named, custom_plate_prepend(names(x)),
+      custom_plate_prepend_full(names(x)))
+    list2env(x, MEMOIZED)
+  }
+  remove_plate_types <- function(x) {
+    x <- c(custom_plate_prepend(x), custom_plate_prepend_full(x))
+    suppressWarnings(rm(list = x, envir = MEMOIZED))
+  }
+  if (!missing(...))
+    warning("arguments other than 'object' are ignored")
+  names(object) <- prepare_names(names(object))
+  nonempty <- vapply(object, length, 0L) > 0L
+  insert_plate_types(object[nonempty])
+  remove_plate_types(names(object)[!nonempty])
+  structure(nonempty, names = custom_plate_prepend(names(object)))
+}, sealed = SEALED)
+
 opm_string <- function(version = FALSE) {
   x <- "opm"
   if (!version)
@@ -254,20 +443,60 @@ clean_plate_positions <- function(x) {
 }
 
 map_well_names <- function(wells, plate, in.parens = FALSE, brackets = FALSE,
-    paren.sep = " ", downcase = FALSE, ...) {
-  pos <- match(L(plate), colnames(WELL_MAP))
-  if (is.na(pos)) {
-    warning("cannot find plate type ", plate)
-    return(trim_string(wells, ...))
+    paren.sep = " ", downcase = FALSE, rm.num = FALSE,
+    max = opm_opt("max.chars"), ...) {
+  if ((L(paren.sep) == "@"))
+    return(sprintf("%s@%s", wells, plate))
+  if (custom_plate_is(plate)) {
+    if (custom_plate_exists(plate))
+      res <- custom_plate_get(plate)[wells]
+    else
+      res <- NULL
+  } else {
+    if (is.na(pos <- match(plate, colnames(WELL_MAP))))
+      res <- NULL
+    else
+      res <- WELL_MAP[wells, pos, "name"]
   }
-  res <- WELL_MAP[wells, pos, "name"]
+  if (is.null(res)) {
+    warning("cannot find plate type ", plate)
+    return(trim_string(str = wells, max = max, ...))
+  }
+  if (rm.num)
+    res <- remove_concentration(res)
   if (downcase)
     res <- substrate_info(res, "downcase")
   if (in.parens)
     add_in_parens(str.1 = wells, str.2 = res, brackets = brackets,
-      paren.sep = paren.sep, ...)
+      paren.sep = paren.sep, max = max, ...)
   else
-    trim_string(str = res, ...)
+    trim_string(str = res, max = max, ...)
+}
+
+well_to_substrate <- function(x, plate) {
+  get_name <- function(x, plate) wells(x, TRUE, FALSE, plate = plate)[, 1L]
+  if (length(plate)) {
+    if (all(grepl(SUBSTRATE_PATTERN[["any"]], x, FALSE, TRUE)))
+      get_name(substr(x, 1L, 3L), plate)
+    else
+      x # assume plain substrate names without wells as prefix
+  } else if (all(grepl("^[A-Z][0-9]{2}@", x, FALSE, TRUE))) {
+    plate <- as.factor(substr(x, 5L, nchar(x)))
+    pos <- split.default(seq_along(x), plate)
+    x <- split.default(substr(x, 1L, 3L), plate)
+    x <- mapply(get_name, x, names(x), SIMPLIFY = FALSE)
+    result <- character(length(plate))
+    for (i in seq_along(x))
+      result[pos[[i]]] <- x[[i]]
+    result
+  } else {
+    for (p in SUBSTRATE_PATTERN[c("paren", "bracket")]) {
+      m <- regexpr(p, x, FALSE, TRUE)
+      if (all(attr(m, "match.length") > 0L))
+        return(get_partial_match(1L, m, x))
+    }
+    x
+  }
 }
 
 to_sentence <- function(x, ...) UseMethod("to_sentence")
@@ -296,37 +525,49 @@ setGeneric("wells", function(object, ...) standardGeneric("wells"))
 
 setMethod("wells", OPM, function(object, full = FALSE, in.parens = TRUE,
     max = opm_opt("max.chars"), brackets = FALSE, clean = TRUE,
-    word.wise = FALSE, paren.sep = " ", downcase = FALSE,
-    plate = plate_type(object)) {
-  result <- setdiff(colnames(measurements(object)), HOUR)
-  if (L(full))
-    map_well_names(result, L(plate), in.parens = in.parens,
+    word.wise = FALSE, paren.sep = " ", downcase = FALSE, rm.num = FALSE,
+    plate = plate_type(object), simplify = TRUE) {
+  LL(full, simplify, plate)
+  x <- colnames(object@measurements)[-1L]
+  if (!missing(plate))
+    plate <- if (custom_plate_is(plate))
+      custom_plate_normalize(plate)
+    else
+      normalize_predefined_plate(plate)
+  if (full)
+    x <- structure(map_well_names(x, plate, in.parens = in.parens,
       max = max, brackets = brackets, clean = clean, word.wise = word.wise,
-      paren.sep = paren.sep, downcase = downcase)
-  else
-    result
+      paren.sep = paren.sep, downcase = downcase, rm.num = rm.num), names = x)
+  if (simplify)
+    return(x)
+  x <- matrix(x, length(x), 1L, FALSE, list(names(x), plate))
+  class(x) <- "well_coords_map"
+  x
 }, sealed = SEALED)
 
 setMethod("wells", "ANY", function(object, full = TRUE, in.parens = FALSE,
     max = opm_opt("max.chars"), brackets = FALSE, clean = TRUE,
-    word.wise = FALSE, paren.sep = " ", downcase = FALSE, plate = "PM01") {
-  result <- well_index(object, rownames(WELL_MAP))
-  if (!is.character(result))
-    result <- rownames(WELL_MAP)[result]
-  result <- do.call(cbind, rep.int(list(result), length(plate)))
-  pos <- pmatch(plate_type(plate), colnames(WELL_MAP))
-  colnames(result) <- plate
-  if (is.character(object))
-    rownames(result) <- object
-  if (!L(full))
-    return(result)
-  for (i in which(!is.na(pos)))
-    result[, i] <- map_well_names(result[, i], colnames(WELL_MAP)[pos[i]],
-      in.parens = in.parens, max = max, brackets = brackets, clean = clean,
-      word.wise = word.wise, paren.sep = paren.sep, downcase = downcase)
-  for (i in which(is.na(pos)))
-    result[, i] <- NA_character_
-  result
+    word.wise = FALSE, paren.sep = " ", downcase = FALSE, rm.num = FALSE,
+    plate = "PM01", simplify = FALSE) {
+  LL(full, simplify)
+  x <- well_index(object, rownames(WELL_MAP))
+  if (!is.character(x))
+    x <- rownames(WELL_MAP)[x]
+  ok <- is.custom <- custom_plate_is(plate)
+  x <- matrix(x, length(x), length(plate), FALSE, list(x, ifelse(is.custom,
+    custom_plate_normalize(plate), normalize_predefined_plate(plate))))
+  ok[is.custom] <- vapply(colnames(x)[is.custom], custom_plate_exists, NA)
+  ok[!is.custom] <- match(colnames(x)[!is.custom], colnames(WELL_MAP), 0L) > 0L
+  x[, !ok] <- NA_character_
+  if (full)
+    for (i in which(ok))
+      x[, i] <- map_well_names(x[, i], colnames(x)[i], in.parens = in.parens,
+        max = max, brackets = brackets, clean = clean, word.wise = word.wise,
+        paren.sep = paren.sep, downcase = downcase, rm.num = rm.num)
+  if (simplify && ncol(x) == 1L)
+    return(x[, 1L])
+  class(x) <- "well_coords_map"
+  x
 }, sealed = SEALED)
 
 setMethod("wells", "missing", function(object, ...) {
@@ -335,6 +576,22 @@ setMethod("wells", "missing", function(object, ...) {
 
 setGeneric("listing")
 
+setOldClass("well_coords_map")
+
+setClass("well_coords_listing", contains = "print_easy")
+
+setMethod("listing", "well_coords_map", function(x) {
+  x <- x[!apply(is.na(x), 1L, all), , drop = FALSE]
+  result <- structure(vector("list", ncol(x)), names = plate <- colnames(x))
+  full <- ifelse(custom_plate_is(plate),
+    mget(custom_plate_prepend_full(custom_plate_proper(plate)), MEMOIZED,
+    "character", rep.int(list(NA_character_), length(plate))), PLATE_MAP[plate])
+  for (i in seq_along(result))
+    result[[i]] <- list(full[[i]], as.list(x[, i]))
+  class(result) <- c("well_coords_listing", "print_easy")
+  result
+}, sealed = SEALED)
+
 setMethod("listing", OPMD, function(x, as.groups,
     cutoff = opm_opt("min.mode"), downcase = TRUE, full = TRUE,
     in.parens = FALSE, html = FALSE, sep = " ", ..., exact = TRUE,
@@ -353,7 +610,7 @@ setMethod("listing", OPMD, function(x, as.groups,
   res
 }, sealed = SEALED)
 
-setMethod("listing", OPMS, function(x, as.groups, cutoff = opm_opt("min.mode"),
+setMethod("listing", XOPMX, function(x, as.groups, cutoff = opm_opt("min.mode"),
     downcase = TRUE, full = TRUE, in.parens = FALSE, html = FALSE, sep = " ",
     ..., exact = TRUE, strict = TRUE) {
   add_stuff <- function(x, html, cutoff) {
@@ -364,7 +621,7 @@ setMethod("listing", OPMS, function(x, as.groups, cutoff = opm_opt("min.mode"),
   }
   LL(cutoff, sep)
   if (!length(as.groups)) {
-    res <- do.call(rbind, lapply(X = x@plates, FUN = listing, html = html,
+    res <- do.call(rbind, lapply(X = plates(x), FUN = listing, html = html,
       downcase = downcase, full = full, in.parens = in.parens,
       as.groups = NULL, ...))
     rownames(res) <- seq_len(nrow(res))
@@ -453,9 +710,10 @@ setGeneric("substrate_info",
   function(object, ...) standardGeneric("substrate_info"))
 
 setMethod("substrate_info", "character", function(object,
-    what = c("cas", "kegg", "drug", "metacyc", "chebi", "mesh", "downcase",
-      "greek", "concentration", "html", "peptide", "all"), browse = 0L,
-    download = FALSE, ...) {
+    what = c("cas", "kegg", "drug", "metacyc", "chebi", "mesh", "seed",
+      "downcase", "greek", "concentration", "html", "peptide", "peptide2",
+      "all"),
+    browse = 0L, download = FALSE, ...) {
 
   find_substrate_id <- function(x) {
     result <- WELL_MAP[, , "substrate_id"][match(x, WELL_MAP[, , "name"])]
@@ -469,9 +727,11 @@ setMethod("substrate_info", "character", function(object,
       chebi = "http://www.ebi.ac.uk/chebi/searchId.do?chebiId=CHEBI:",
       metacyc = "http://biocyc.org/META/NEW-IMAGE?type=COMPOUND&object=",
       cas = "http://chem.sis.nlm.nih.gov/chemidplus/direct.jsp?regno=",
-      mesh = "http://www.ncbi.nlm.nih.gov/mesh/"
+      mesh = "http://www.ncbi.nlm.nih.gov/mesh/",
+      seed = paste0("http://seed-viewer.theseed.org/seedviewer.cgi?",
+        "page=CompoundViewer&compound=")
     )
-    base <- url_base[match.arg(how, names(url_base))]
+    base <- url_base[[match.arg(how, names(url_base))]]
     x <- sub("^(CAS\\s+|CHEBI:)", "", x, TRUE, TRUE)
     ifelse(is.na(x), NA_character_, paste0(base, vapply(x, URLencode, "")))
   }
@@ -508,20 +768,42 @@ setMethod("substrate_info", "character", function(object,
   }
 
   extract_concentration <- function(x) {
-    # TODO: maybe extraction method for "mM" and "%" data also useful
-    x <- ifelse(grepl(SUBSTRATE_PATTERN[["either"]], x, FALSE, TRUE),
-      substr(x, 1L, nchar(x) - 1L), x)
+    in.parens <- grepl(SUBSTRATE_PATTERN[["either"]], x, FALSE, TRUE)
+    x <- ifelse(in.parens, substr(x, 1L, nchar(x) - 1L), x)
     m <- regexpr("(?<=#)\\s*\\d+\\s*$", x, FALSE, TRUE)
+    ## The following code is currently not in use because the only plate to
+    ## which it is applicable (PM09) does not show regularity anyway. Conversion
+    ## to integer would also be problematic because contractions such as 5.5 or
+    ## 6.5 are present.
+    #if (all(m < 0L)) {
+    #  x <- ifelse(in.parens, substr(x, 6L, nchar(x)), x)
+    #  m <- regexpr("^(?:\\d+(?:\\.\\d+)?)(?=%|mM)", x, FALSE, TRUE)
+    #}
     as.integer(substr(x, m, m + attr(m, "match.length") - 1L))
   }
 
-  parse_peptide <- function(x) {
-    parse <- function(x) strsplit(gsub("(?<=\\b[A-Za-z])-", "_", x, FALSE,
-      TRUE), "-", TRUE)
+  parse_peptide <- function(x, remove.L) {
+    recognize_full_names <- function(x) {
+      m <- regexpr("^(?:[A-Za-z][,-])*[A-Za-z]-", x, FALSE, TRUE)
+      result <- AMINO_ACIDS[substr(x, m + attr(m, "match.length"), nchar(x))]
+      ok <- !is.na(result)
+      prefix <- m > 0L & ok
+      m <- substr(x, m, m + attr(m, "match.length") - 1L)
+      result[prefix] <- paste0(m[prefix], result[prefix])
+      result <- as.list(result)
+      result[!ok] <- list(character())
+      result
+    }
     result <- structure(vector("list", length(x)), names = x)
-    pat <- sprintf("^%s(-%s)*$", pat <- "([A-Za-z]-)?[A-Z][a-z][a-z]", pat)
-    result[ok] <- parse(x[ok <- grepl(pat, x, FALSE, TRUE)])
-    result[!ok] <- list(character())
+    x <- remove_concentration(x)
+    pat <- "(([A-Za-z][,-])*[A-Za-z]-)?[A-Z][a-z]{2}"
+    pat <- sprintf("^%s(-%s)*$", pat, pat)
+    ok <- grepl(pat, x, FALSE, TRUE)
+    result[ok] <- strsplit(x[ok], "(?<!\\b\\w)-", FALSE, TRUE)
+    result[!ok] <- recognize_full_names(x[!ok])
+    if (remove.L)
+      result <- lapply(result, sub, pattern = "^L-", replacement = "",
+        ignore.case = FALSE, perl = TRUE)
     result
   }
 
@@ -538,13 +820,14 @@ setMethod("substrate_info", "character", function(object,
 
   result <- case(what <- match.arg(what),
     all = all_information(object),
-    chebi =, drug =, kegg =, metacyc =, mesh =,
+    chebi =, drug =, kegg =, metacyc =, mesh =, seed =,
     cas = SUBSTRATE_INFO[find_substrate_id(object), toupper(what)],
     concentration = extract_concentration(object),
     downcase = safe_downcase(object),
     greek = expand_greek_letters(object),
     html = compound_name_to_html(object),
-    peptide = parse_peptide(object)
+    peptide = parse_peptide(object, TRUE),
+    peptide2 = parse_peptide(object, FALSE)
   )
   browse <- must(as.integer(L(browse)))
   if (browse != 0L) {
@@ -606,9 +889,9 @@ web_query <- function(ids, what = c("kegg", "drug")) {
 NULL
 
 collect.kegg_compounds <- function(x,
-    what = c("pathway", "brite", "activity", "exact_mass"), min.cov = 2L,
+    what = c("pathway", "brite", "activity", "exact_mass"),
     missing.na = TRUE, ...) {
-  partial_matrix <- function(name, x, min.cov) {
+  partial_matrix <- function(name, x) {
     convert <- list(
       ACTIVITY = function(x) {
         # notes in brackets make entries more specific; we use both variants
@@ -633,10 +916,10 @@ collect.kegg_compounds <- function(x,
     if (name == "EXACT_MASS")
       matrix(unlist(result), ncol = 1L, dimnames = list(NULL, tolower(name)))
     else
-      pkgutils::collect(result, "occurrences", min.cov)
+      pkgutils::collect(result, "occurrences")
   }
   what <- toupper(match.arg(what, several.ok = TRUE))
-  result <- do.call(cbind, lapply(what, partial_matrix, x, min.cov))
+  result <- do.call(cbind, lapply(what, partial_matrix, x))
   if (L(missing.na))
     result[!vapply(x, length, 0L), ] <- as(NA, typeof(result))
   result
@@ -667,3 +950,15 @@ lapply(c(
   }, sealed = SEALED)
 })
 
+lapply(c(
+    #+
+    find_positions,
+    substrate_info,
+    wells
+    #-
+  ), FUN = function(func_) {
+  setMethod(func_, MOPMX, function(object, ...) {
+    lapply(object@.Data, FUN = func_, ...)
+  }, sealed = SEALED)
+})
+
