@@ -56,6 +56,16 @@ test_that("plates can be merged", {
   expect_equal(dim(x), c(dim(y[1L])[1L] + dim(y[2L])[1L], d1[3L]))
 })
 
+## merge
+test_that("MOPMX objects can be merged", {
+  got <- merge(MOPMX.1)
+  expect_equal(got, MOPMX.1)
+  got <- merge(MOPMX.1, MOPMX.1[[2]])
+  expect_equal(length(got), length(MOPMX.1))
+  expect_true(length(plates(got)) > length(plates(MOPMX.1)))
+  expect_error(got <- merge(MOPMX.2)) # non-uniform wells
+})
+
 
 ## merge
 test_that("character-matrix objects can be merged", {
@@ -84,7 +94,12 @@ test_that("character-matrix objects can be merged", {
 
 
 ## split
-## UNTESTED
+test_that("MOPMX objects can be split", {
+  expect_warning(got <- split(MOPMX.1, "organism"))
+  expect_is(got, "list")
+  expect_equal(names(got), c(ORGN, "NA"))
+  expect_true(all(vapply(got, is, NA, MOPMX)))
+})
 
 
 ## plates
@@ -95,9 +110,20 @@ test_that("the plates can be obtained as a list", {
   expect_true(all(vapply(pl, is, NA, "OPM")))
 })
 
+## plates
+test_that("the plates in MOPMX objects can be obtained as a list", {
+  pl <- plates(MOPMX.1)
+  expect_is(pl, "list")
+  expect_equal(length(pl), 3L)
+  expect_true(all(vapply(pl, is, NA, "OPM")))
+})
+
 
 ## oapply
-## UNTESTED -- but see examples and tests for subsetting
+test_that("oapply() can be applied to MOPMX objects", {
+  got <- oapply(MOPMX.1, identity)
+  expect_equal(got, MOPMX.1)
+})
 
 
 ################################################################################
@@ -110,7 +136,7 @@ test_that("example data can be flattened", {
   expect_is(flat, "data.frame")
   expect_equal(colnames(flat), base.colnames)
   w <- wells(SMALL, full = TRUE)
-  expect_equal(as.character(unique(flat$Well)), w)
+  expect_equivalent(as.character(unique.default(flat$Well)), w)
   exp.len <- Reduce(`*`, dim(SMALL))
   expect_equal(exp.len, nrow(flat))
 })
@@ -173,6 +199,18 @@ test_that("OPMS objects including metadata can be flattened", {
 })
 
 
+## flatten
+test_that("MOPMX objects can be flattened", {
+  got <- flatten(MOPMX.1)
+  expect_true(setequal(got[, CSV_NAMES[["PLATE_TYPE"]]], plate_type(MOPMX.1)))
+  expect_equal(length(unique.default(got[, RESERVED_NAMES[["plate"]]])),
+    max(vapply(MOPMX.1, length, 0L)))
+  got <- interaction(got[,
+    c(RESERVED_NAMES[["plate"]], CSV_NAMES[["PLATE_TYPE"]])], drop = TRUE)
+  expect_equal(length(levels(got)), length(plates(MOPMX.1)))
+})
+
+
 ## flattened_to_factor
 ## UNTESTED
 
@@ -410,11 +448,25 @@ test_that("extract_columns can be applied to a data frame", {
 
 
 ## sort
-## UNTESTED
+test_that("MOPMX objects can be sorted", {
+  got <- sort(MOPMX.1)
+  expect_equal(got, rev(MOPMX.1))
+  got <- sort(MOPMX.1, by = "length")
+  expect_equal(got, MOPMX.1)
+})
 
 
 ## unique
-## UNTESTED
+test_that("MOPMX objects can be made unique", {
+  got <- unique(MOPMX.1)
+  expect_equal(got, MOPMX.1)
+  got <- unique(MOPMX.1 + OPM.3)
+  expect_equal(got, MOPMX.1)
+  got <- unique(MOPMX.1 + OPM.1, what = "all")
+  expect_true(length(got) > length(MOPMX.1))
+  got <- unique(MOPMX.1 + OPM.1, what = "plate.type")
+  expect_equal(got, MOPMX.1)
+})
 
 
 ## rep
@@ -439,7 +491,7 @@ test_that("aggregated parameters can be extracted as matrix", {
   mat <- extract(THIN.AGG, as.labels = list("organism", "run"), sep = "||")
   expect_is(mat, "matrix")
   expect_equal(dim(mat), c(2L, 96L))
-  expect_equal(colnames(mat), wells(THIN.AGG, full = TRUE))
+  expect_equivalent(colnames(mat), wells(THIN.AGG, full = TRUE))
   expect_equal(NULL, attr(mat, "row.groups"))
   expect_equal(rn, rownames(mat))
 
@@ -447,7 +499,7 @@ test_that("aggregated parameters can be extracted as matrix", {
     subset = "lambda", as.groups = list("run", "organism"), sep = "||")
   expect_is(mat, "matrix")
   expect_equal(dim(mat), c(2L, 96L))
-  expect_equal(colnames(mat), wells(THIN.AGG, full = TRUE))
+  expect_equivalent(colnames(mat), wells(THIN.AGG, full = TRUE))
   expect_equal(as.factor(gn), attr(mat, "row.groups"))
   expect_equal(rn, rownames(mat))
 
@@ -477,7 +529,7 @@ test_that("aggregated parameters can be extracted as matrix with CIs", {
   expect_is(mat, "matrix")
   expect_is(mat[1L], "numeric")
   expect_equal(dim(mat), c(6L, 96L))
-  expect_equal(colnames(mat), wells(THIN.AGG, full = TRUE))
+  expect_equivalent(colnames(mat), wells(THIN.AGG, full = TRUE))
   expect_equal(grepl(rn[1L], rownames(mat), fixed = TRUE), c(T, T, T, F, F, F))
   expect_equal(grepl(rn[2L], rownames(mat), fixed = TRUE), c(F, F, F, T, T, T))
   expect_equal(NULL, attr(mat, "row.groups"))
@@ -487,7 +539,7 @@ test_that("aggregated parameters can be extracted as matrix with CIs", {
   expect_is(mat, "matrix")
   expect_is(mat[1L], "numeric")
   expect_equal(dim(mat), c(6L, 96L))
-  expect_equal(colnames(mat), wells(THIN.AGG, full = TRUE))
+  expect_equivalent(colnames(mat), wells(THIN.AGG, full = TRUE))
   expect_equal(grepl(rn[1L], rownames(mat), fixed = TRUE), c(T, T, T, F, F, F))
   expect_equal(grepl(rn[2L], rownames(mat), fixed = TRUE), c(F, F, F, T, T, T))
   expect_equal(rep(as.factor(metadata(THIN.AGG, "organism")), each = 3L),
@@ -503,7 +555,7 @@ test_that("aggregated parameters can be extracted as dataframe", {
     subset = "lambda", dataframe = TRUE, sep = "***")
   expect_is(mat, "data.frame")
   expect_equal(dim(mat), c(2L, 99L))
-  expect_equal(colnames(mat), c("organism", "run",
+  expect_equivalent(colnames(mat), c("organism", "run",
     RESERVED_NAMES[["parameter"]],
     wells(THIN.AGG, full = TRUE)))
   expect_true(all(vapply(mat[, 1L:3L], is.factor, NA)))
@@ -517,7 +569,7 @@ test_that("aggregated parameters can be extracted as dataframe", {
     as.groups = list("run", "organism"))
   expect_is(mat, "data.frame")
   expect_equal(dim(mat), c(2L, 101L))
-  expect_equal(colnames(mat), c("organism", "run",
+  expect_equivalent(colnames(mat), c("organism", "run",
     RESERVED_NAMES[["parameter"]],
     wells(THIN.AGG, full = TRUE), "run", "organism"))
   expect_true(all(vapply(mat[, 1L:3L], is.factor, NA)))
@@ -539,7 +591,7 @@ test_that("aggregated parameters can be extracted as dataframe with CIs", {
     subset = "lambda", dataframe = TRUE, sep = "***", ci = TRUE)
   expect_is(mat, "data.frame")
   expect_equal(dim(mat), c(6L, 99L))
-  expect_equal(colnames(mat), c("organism", "run",
+  expect_equivalent(colnames(mat), c("organism", "run",
     RESERVED_NAMES[["parameter"]],
     wells(THIN.AGG, full = TRUE)))
   expect_true(all(vapply(mat[, 1L:3L], is.factor, NA)))
@@ -564,7 +616,7 @@ test_that("aggregated parameters can be extracted as dataframe with CIs", {
     as.groups = ~ run - organism)
   expect_is(mat, "data.frame")
   expect_equal(dim(mat), c(6L, 101L))
-  expect_equal(colnames(mat), c("organism", "run",
+  expect_equivalent(colnames(mat), c("organism", "run",
     RESERVED_NAMES[["parameter"]],
     wells(THIN.AGG, full = TRUE), "run", "organism"))
   expect_true(all(vapply(mat[, 1L:3L], is.factor, NA)))
@@ -582,7 +634,9 @@ test_that("aggregated parameters can be extracted as dataframe with CIs", {
     subset = "lambda", dataframe = TRUE, sep = "***", ci = TRUE,
     as.groups = ~ J(run - organism))
   expect_equal(dim(mat2), c(6L, 102L))
-  expect_equivalent(mat, mat2[, -102L]) # TODO: unclear why different
+  # the following subsetting itself causes differences in column names
+  expect_equivalent(mat, mat2[, -102L])
+  expect_equal(colnames(mat), colnames(mat2)[-102L])
 
 })
 
@@ -696,11 +750,39 @@ test_that("extract() deals with duplicate column names", {
 })
 
 
+## extract
+test_that("extract() can be applied to MOPMX objects", {
+
+  # matrices
+  got <- extract(MOPMX.2, ~ run, dataframe = FALSE, as.groups = "organism")
+  expect_is(got, "matrix")
+  expect_equal(mode(got), "numeric")
+  expect_true(setequal(to_metadata(MOPMX.2)$run, rownames(got)))
+  expect_false(all(to_metadata(MOPMX.2)$run == rownames(got)))
+  expect_equal(nrow(got), sum(vapply(MOPMX.2, length, 0L)))
+  expect_equal(sum(complete.cases(got)), 2L)
+  expect_equal(length(attr(got, "row.groups")),
+    sum(vapply(MOPMX.2, length, 0L)))
+
+  # data frames
+  got <- extract(MOPMX.2, ~ run, dataframe = TRUE, as.groups = "organism")
+  expect_is(got, "data.frame")
+  expect_equal(colnames(got)[ncol(got)], "organism")
+  expect_equal(sum(complete.cases(got)), 2L)
+  expect_equal(nrow(got), sum(vapply(MOPMX.2, length, 0L)))
+  expect_true(all(to_metadata(MOPMX.2)$run == got$run))
+
+})
+
+
 ################################################################################
 
 
 ## as.data.frame
-## UNTESTED
+test_that("MOPMX objects can be converted into a data frame", {
+  x <- as.data.frame(MOPMX.1)
+  expect_equal(dim(x), c(288L, 5L))
+})
 
 
 ################################################################################
@@ -790,11 +872,30 @@ test_that("OPMS example data can be converted to YAML", {
   expect_equal("---", lines[1L])
   pats <- c("metadata", "measurements", "csv_data", "aggregated")
   for (pat in sprintf("^[ -] %s:$", pats)) {
-    pos <- grep(pat, lines, perl = TRUE)
+    pos <- grep(pat, lines, FALSE, TRUE)
     expect_equal(length(THIN.AGG), length(pos))
   }
 })
 
+## to_yaml
+test_that("MOPMX example data can be converted to YAML", {
+  lines <- strsplit(to_yaml(MOPMX.1), "\n", fixed = TRUE)[[1]]
+  expect_equal("---", lines[1L])
+  pats <- c("measurements", "csv_data")
+  for (pat in sprintf("^[ -] %s:$", pats)) {
+    pos <- grep(pat, lines, FALSE, TRUE)
+    expect_equal(length(pos), 3L)
+  }
+})
+
+
+
+################################################################################
+
+
+## opmx
+## UNTESTED
+
 
 ################################################################################
 
